#!/usr/bin/env node

var fs = require('fs');
var path = require('path');
var _ = require("underscore");
var chalk = require('chalk');
var when = require('when');
var multiline = require('multiline');
var request = require("request");
var shell = require("shelljs");
var ezreq = require("easy-request");
var read = require("read");
var daemon = require("daemon");

var usage = multiline(function() {
	/*

  Usage: kite <command>

    Commands:

      install [name]  make a kite
      status          query kite's status
      update          update kite
      fly             fly the kite!
      stop            pull back the kite
      help
*/
});

function help() {
	console.log(usage);
	console.log();
}

function execSync(command) {
	console.log(">", command);
	if (shell.exec(command).code !== 0) {
		throw new Error("fail to execute " + command);
	}
}

var argv = process.argv.slice(2);
if (argv.length <= 0) {
	help();
	process.exit(-1);
}
var command = argv[0]

function status() {
	var port = JSON.parse(fs.readFileSync(".kite.json")).port;

	if (!port) {
		return when.promise(function(resolve, reject) {
			resolve(null);
		});
	}

	return ezreq.get("http://localhost:" + port + "/status").then(function(data) {
		return data;
	}, function() {
		return null;
	});
}

function stop() {
	var port = JSON.parse(fs.readFileSync(".kite.json")).port;

	if (!port) {
		return when.promise(function(resolve, reject) {
			resolve(null);
		});
	}

	return ezreq.post("http://localhost:" + port + "/stop");
}

function authenticate(username, password) {
	if (process.env.ENV === 'development') {
		var deferred = when.defer();
		process.nextTick(function() {
			deferred.resolve({
				access_token: '3abe4d187a3a40ebaa4ac3fe53463f2f',
				token: '8798'
			});
		});
		return deferred.promise;
	}

	return ezreq.post("http://www.kuzhanggui.com/backend/auth.json", {
		username: username,
		password: password
	}).then(function(data) {
		if (data.ret_code !== 0) {
			throw new Error("Authentication failed");
		}

		return data;
	});
}

function install() {
	var args = require("minimist")(process.argv.slice(3), {
		string: ["username", "password", "host"],
		alias: {
			"u": "username",
			"p": "password",
			"h": "host"
		}
	});

	var name = args._.length > 0 ? args._[0] : "kite"
	var dir = path.join(process.cwd(), name);
	try {
		if (fs.statSync(dir).isDirectory()) {
			return console.log(chalk.red(name + " exsits"));
		}
	} catch (e) {}


	function ensurePrompt(value, prompt, silent) {
		var deferred = when.defer();
		if (value) {
			console.log(value);
			process.nextTick(function() {
				deferred.resolve(value);
			});
		} else {
			read({
				prompt: prompt,
				silent: silent
			}, function(err, value) {
				if (err) {
					deferred.reject(err);
				} else {
					deferred.resolve(value);
				}
			});
		}

		return deferred.promise;
	}

	var username, password, host;
	ensurePrompt(args.username, "Username: ").then(function(_username) {
		username = _username;
		return ensurePrompt(args.password, "Password: ", true);
	}).then(function(_password) {
		password = _password;
		return ensurePrompt(args.host, "Host: ");
	}).then(function(_host) {
		host = _host;

		// TODO validate host
		if (!username || !password || !host) {
			return console.log(chalk.red("Username, password or host is invalid!"));
		}

		authenticate(username, password).then(function(result) {
			execSync("git clone git@github.com:limijiaoyin/kite.git " + name);
			fs.writeFileSync(path.join(dir, ".kite.json"), JSON.stringify({
				server: "http://www.kuzhanggui.com",
				site: host,
				access_token: result.access_token,
				token: result.token
			}, null, 2));
			execSync("cd " + name + " && npm install --production");
			console.log(chalk.green("Install successfully!"));
		}).catch(function(e) {
			console.log(chalk.red("Fail to install"));
			console.trace(e);
		});
	}, function(err) {
		console.log(chalk.red("Fail to install."));
	});
}

if (command === 'status') {
	status().then(function(status) {
		if (status) {
			console.log(chalk.green("kite is running"));
			console.log(chalk.green(JSON.stringify(status, null, 2)));
		} else {
			console.log(chalk.red("kite is stopped"));
		}
	});
} else if (command === "fly" || command === "start") {
	var args = require("minimist")(argv.slice(1), {
		alias: {
			p: "port"
		}
	});

	status().then(function(status) {
		if (status) {
			console.log(chalk.green("Kite is already running!"));
		} else {
			var script = path.join(process.cwd(), "index.js");
			if (_.isNumber(args.port)) {
				daemon.daemon(script, ["-p", args.port]);
			} else {
				daemon.daemon(script, []);
			}
			console.log(chalk.green("Start successfully!"));
		}
	});
} else if (command === "stop") {
	stop().then(function() {
		console.log(chalk.green("Stop successfully!"));
	}, function(err) {
		console.log(chalk.red("Fail to stop kite"));
		console.trace(err);
	});
} else if (command === 'update') {
	status().then(function(status) {
		console.log("Kite is runnning, stop it...");
		if (status) {
			return stop();
		}
	}).then(function() {
		console.log("Kite is stopped.");
		execSync("git pull origin");
		execSync("npm install --production");
		console.log(chalk.green("Update successfully!"));
	}).catch(function(err) {
		console.log(chalk.red("Fail to update"));
		throw new Error(err);
	});

} else if (command === 'install') {
	install();
} else {
	help();
}